@@ -92,7 +92,9 @@ zend_function_entry redis_array_functions[] = {
      PHP_FE_END
 };
 
-static void redis_array_free(RedisArray *ra) {
+static void
+redis_array_free(RedisArray *ra)
+{
     int i;
 
     /* Redis objects */
@@ -116,97 +118,106 @@ static void redis_array_free(RedisArray *ra) {
     efree(ra);
 }
 
-int le_redis_array;
-void redis_destructor_redis_array(zend_resource * rsrc TSRMLS_DC)
+#if (PHP_MAJOR_VERSION < 7)
+typedef struct {
+    zend_object std;
+    RedisArray *ra;
+} redis_array_object;
+
+void
+free_redis_array_object(void *object TSRMLS_DC)
 {
-    RedisArray *ra = (RedisArray*)rsrc->ptr;
+    redis_array_object *obj = (redis_array_object *)object;
 
-    /* Free previous ring if it's set */
-    if(ra->prev) redis_array_free(ra->prev);
-
-    /* Free parent array */
-    redis_array_free(ra);
+    zend_object_std_dtor(&obj->std TSRMLS_CC);
+    if (obj->ra) {
+        if (obj->ra->prev) redis_array_free(obj->ra->prev);
+        redis_array_free(obj->ra);
+    }
+    efree(obj);
 }
 
+zend_object_value
+create_redis_array_object(zend_class_entry *ce TSRMLS_DC)
+{
+    zend_object_value retval;
+    redis_array_object *obj = ecalloc(1, sizeof(redis_array_object));
+    memset(obj, 0, sizeof(redis_array_object));
+
+    zend_object_std_init(&obj->std, ce TSRMLS_CC);
+
+#if PHP_VERSION_ID < 50399
+    zval *tmp;
+    zend_hash_copy(obj->std.properties, &ce->default_properties,
+        (copy_ctor_func_t)zval_add_ref, (void *)&tmp, sizeof(zval *));
+#endif
+
+    retval.handle = zend_objects_store_put(obj,
+        (zend_objects_store_dtor_t)zend_objects_destroy_object,
+        (zend_objects_free_object_storage_t)free_redis_array_object,
+        NULL TSRMLS_CC);
+    retval.handlers = zend_get_std_object_handlers();
+
+    return retval;
+}
+#else
+typedef struct {
+    RedisArray *ra;
+    zend_object std;
+} redis_array_object;
+
+zend_object_handlers redis_array_object_handlers;
+
+void
+free_redis_array_object(zend_object *object)
+{
+    redis_array_object *obj = (redis_array_object *)((char *)(object) - XtOffsetOf(redis_array_object, std));
+
+    if (obj->ra) {
+        if (obj->ra->prev) redis_array_free(obj->ra->prev);
+        redis_array_free(obj->ra);
+    }
+    zend_object_std_dtor(&obj->std TSRMLS_CC);
+}
+
+zend_object *
+create_redis_array_object(zend_class_entry *ce TSRMLS_DC)
+{
+    redis_array_object *obj = ecalloc(1, sizeof(redis_array_object) + zend_object_properties_size(ce));
+
+    obj->ra = NULL;
+
+    zend_object_std_init(&obj->std, ce TSRMLS_CC);
+    object_properties_init(&obj->std, ce);
+
+    memcpy(&redis_array_object_handlers, zend_get_std_object_handlers(), sizeof(redis_array_object_handlers));
+    redis_array_object_handlers.offset = XtOffsetOf(redis_array_object, std);
+    redis_array_object_handlers.free_obj = free_redis_array_object;
+    obj->std.handlers = &redis_array_object_handlers;
+
+    return &obj->std;
+}
+#endif
 
 /**
  * redis_array_get
  */
-PHP_REDIS_API int redis_array_get(zval *id, RedisArray **ra TSRMLS_DC)
+PHP_REDIS_API RedisArray *
+redis_array_get(zval *id TSRMLS_DC)
 {
+    redis_array_object *obj;
 
-    zval *socket;
-
-    if (Z_TYPE_P(id) != IS_OBJECT || (socket = zend_hash_str_find(Z_OBJPROP_P(id),
-            "socket", sizeof("socket") - 1)) == NULL) {
-        return -1;
-    }
-
+    if (Z_TYPE_P(id) == IS_OBJECT) {
 #if (PHP_MAJOR_VERSION < 7)
-    int resource_type;
-    *ra = (RedisArray *)zend_list_find(Z_LVAL_P(socket), &resource_type);
-    if (!*ra || resource_type != le_redis_array) {
+        obj = (redis_array_object *)zend_objects_get_address(id TSRMLS_CC);
 #else
-    if (Z_RES_P(socket) == NULL ||
-        !(*ra = (RedisArray *)Z_RES_P(socket)->ptr) ||
-        Z_RES_P(socket)->type != le_redis_array
-    ) {
+        obj = (redis_array_object *)((char *)Z_OBJ_P(id) - XtOffsetOf(redis_array_object, std));
 #endif
-        return -1;
+        if (obj->ra) {
+            return obj->ra;
+        }
     }
-
-    return 0;
-}
-
-uint32_t rcrc32(const char *s, size_t sz) {
-
-	static const uint32_t table[256] = {
-		0x00000000,0x77073096,0xEE0E612C,0x990951BA,0x076DC419,0x706AF48F,0xE963A535,
-		0x9E6495A3,0x0EDB8832,0x79DCB8A4,0xE0D5E91E,0x97D2D988,0x09B64C2B,0x7EB17CBD,
-		0xE7B82D07,0x90BF1D91,0x1DB71064,0x6AB020F2,0xF3B97148,0x84BE41DE,0x1ADAD47D,
-		0x6DDDE4EB,0xF4D4B551,0x83D385C7,0x136C9856,0x646BA8C0,0xFD62F97A,0x8A65C9EC,
-		0x14015C4F,0x63066CD9,0xFA0F3D63,0x8D080DF5,0x3B6E20C8,0x4C69105E,0xD56041E4,
-		0xA2677172,0x3C03E4D1,0x4B04D447,0xD20D85FD,0xA50AB56B,0x35B5A8FA,0x42B2986C,
-		0xDBBBC9D6,0xACBCF940,0x32D86CE3,0x45DF5C75,0xDCD60DCF,0xABD13D59,0x26D930AC,
-		0x51DE003A,0xC8D75180,0xBFD06116,0x21B4F4B5,0x56B3C423,0xCFBA9599,0xB8BDA50F,
-		0x2802B89E,0x5F058808,0xC60CD9B2,0xB10BE924,0x2F6F7C87,0x58684C11,0xC1611DAB,
-		0xB6662D3D,0x76DC4190,0x01DB7106,0x98D220BC,0xEFD5102A,0x71B18589,0x06B6B51F,
-		0x9FBFE4A5,0xE8B8D433,0x7807C9A2,0x0F00F934,0x9609A88E,0xE10E9818,0x7F6A0DBB,
-		0x086D3D2D,0x91646C97,0xE6635C01,0x6B6B51F4,0x1C6C6162,0x856530D8,0xF262004E,
-		0x6C0695ED,0x1B01A57B,0x8208F4C1,0xF50FC457,0x65B0D9C6,0x12B7E950,0x8BBEB8EA,
-		0xFCB9887C,0x62DD1DDF,0x15DA2D49,0x8CD37CF3,0xFBD44C65,0x4DB26158,0x3AB551CE,
-		0xA3BC0074,0xD4BB30E2,0x4ADFA541,0x3DD895D7,0xA4D1C46D,0xD3D6F4FB,0x4369E96A,
-		0x346ED9FC,0xAD678846,0xDA60B8D0,0x44042D73,0x33031DE5,0xAA0A4C5F,0xDD0D7CC9,
-		0x5005713C,0x270241AA,0xBE0B1010,0xC90C2086,0x5768B525,0x206F85B3,0xB966D409,
-		0xCE61E49F,0x5EDEF90E,0x29D9C998,0xB0D09822,0xC7D7A8B4,0x59B33D17,0x2EB40D81,
-		0xB7BD5C3B,0xC0BA6CAD,0xEDB88320,0x9ABFB3B6,0x03B6E20C,0x74B1D29A,0xEAD54739,
-		0x9DD277AF,0x04DB2615,0x73DC1683,0xE3630B12,0x94643B84,0x0D6D6A3E,0x7A6A5AA8,
-		0xE40ECF0B,0x9309FF9D,0x0A00AE27,0x7D079EB1,0xF00F9344,0x8708A3D2,0x1E01F268,
-		0x6906C2FE,0xF762575D,0x806567CB,0x196C3671,0x6E6B06E7,0xFED41B76,0x89D32BE0,
-		0x10DA7A5A,0x67DD4ACC,0xF9B9DF6F,0x8EBEEFF9,0x17B7BE43,0x60B08ED5,0xD6D6A3E8,
-		0xA1D1937E,0x38D8C2C4,0x4FDFF252,0xD1BB67F1,0xA6BC5767,0x3FB506DD,0x48B2364B,
-		0xD80D2BDA,0xAF0A1B4C,0x36034AF6,0x41047A60,0xDF60EFC3,0xA867DF55,0x316E8EEF,
-		0x4669BE79,0xCB61B38C,0xBC66831A,0x256FD2A0,0x5268E236,0xCC0C7795,0xBB0B4703,
-		0x220216B9,0x5505262F,0xC5BA3BBE,0xB2BD0B28,0x2BB45A92,0x5CB36A04,0xC2D7FFA7,
-		0xB5D0CF31,0x2CD99E8B,0x5BDEAE1D,0x9B64C2B0,0xEC63F226,0x756AA39C,0x026D930A,
-		0x9C0906A9,0xEB0E363F,0x72076785,0x05005713,0x95BF4A82,0xE2B87A14,0x7BB12BAE,
-		0x0CB61B38,0x92D28E9B,0xE5D5BE0D,0x7CDCEFB7,0x0BDBDF21,0x86D3D2D4,0xF1D4E242,
-		0x68DDB3F8,0x1FDA836E,0x81BE16CD,0xF6B9265B,0x6FB077E1,0x18B74777,0x88085AE6,
-		0xFF0F6A70,0x66063BCA,0x11010B5C,0x8F659EFF,0xF862AE69,0x616BFFD3,0x166CCF45,
-		0xA00AE278,0xD70DD2EE,0x4E048354,0x3903B3C2,0xA7672661,0xD06016F7,0x4969474D,
-		0x3E6E77DB,0xAED16A4A,0xD9D65ADC,0x40DF0B66,0x37D83BF0,0xA9BCAE53,0xDEBB9EC5,
-		0x47B2CF7F,0x30B5FFE9,0xBDBDF21C,0xCABAC28A,0x53B39330,0x24B4A3A6,0xBAD03605,
-		0xCDD70693,0x54DE5729,0x23D967BF,0xB3667A2E,0xC4614AB8,0x5D681B02,0x2A6F2B94,
-		0xB40BBE37,0xC30C8EA1,0x5A05DF1B,0x2D02EF8D};
-
-	unsigned long ret = 0xffffffff;
-	size_t i;
-
-	for (i = 0; i < sz; i++) {
-		ret = (ret >> 8) ^ table[ (ret ^ ((unsigned char)s[i])) & 0xFF ];
-	}
-	return (ret ^ 0xFFFFFFFF);
-
+    return NULL;
 }
 
 /* {{{ proto RedisArray RedisArray::__construct()
@@ -219,7 +230,8 @@ PHP_METHOD(RedisArray, __construct)
 	HashTable *hPrev = NULL, *hOpts = NULL;
 	long l_retry_interval = 0;
   	zend_bool b_lazy_connect = 0;
-	double d_connect_timeout = 0;
+	double d_connect_timeout = 0, read_timeout = 0.0;
+    redis_array_object *obj;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z|a", &z0, &z_opts) == FAILURE) {
 		RETURN_FALSE;
@@ -288,6 +300,17 @@ PHP_METHOD(RedisArray, __construct)
                 d_connect_timeout = atof(Z_STRVAL_P(zpData));
             }
         }
+
+        /* extract read_timeout option */
+        if ((zpData = zend_hash_str_find(hOpts, "read_timeout", sizeof("read_timeout") - 1)) != NULL) {
+            if (Z_TYPE_P(zpData) == IS_DOUBLE) {
+                read_timeout = Z_DVAL_P(zpData);
+            } else if (Z_TYPE_P(zpData) == IS_LONG) {
+                read_timeout = Z_LVAL_P(zpData);
+            } else if (Z_TYPE_P(zpData) == IS_STRING) {
+                read_timeout = atof(Z_STRVAL_P(zpData));
+            }
+        }
 	}
 
 	/* extract either name of list of hosts from z0 */
@@ -297,12 +320,11 @@ PHP_METHOD(RedisArray, __construct)
 			break;
 
 		case IS_ARRAY:
-			ra = ra_make_array(Z_ARRVAL_P(z0), &z_fun, &z_dist, hPrev, b_index, b_pconnect, l_retry_interval, b_lazy_connect, d_connect_timeout TSRMLS_CC);
+			ra = ra_make_array(Z_ARRVAL_P(z0), &z_fun, &z_dist, hPrev, b_index, b_pconnect, l_retry_interval, b_lazy_connect, d_connect_timeout, read_timeout TSRMLS_CC);
 			break;
 
 		default:
 			WRONG_PARAM_COUNT;
-			break;
 	}
     zval_dtor(&z_dist);
     zval_dtor(&z_fun);
@@ -312,24 +334,18 @@ PHP_METHOD(RedisArray, __construct)
 		ra->connect_timeout = d_connect_timeout;
 		if(ra->prev) ra->prev->auto_rehash = b_autorehash;
 #if (PHP_MAJOR_VERSION < 7)
-        int id;
-#if PHP_VERSION_ID >= 50400
-		id = zend_list_insert(ra, le_redis_array TSRMLS_CC);
+        obj = (redis_array_object *)zend_objects_get_address(getThis() TSRMLS_CC);
 #else
-		id = zend_list_insert(ra, le_redis_array);
-#endif
-		add_property_resource(getThis(), "socket", id);
-#else
-        zval *id = zend_list_insert(ra, le_redis_array TSRMLS_CC);
-        add_property_resource(getThis(), "socket", Z_RES_P(id));
+        obj = (redis_array_object *)((char *)Z_OBJ_P(getThis()) - XtOffsetOf(redis_array_object, std));
 #endif
+        obj->ra = ra;
 	}
 }
 
 static void
 ra_forward_call(INTERNAL_FUNCTION_PARAMETERS, RedisArray *ra, const char *cmd, int cmd_len, zval *z_args, zval *z_new_target) {
 
-	zval z_tmp, z_fun, *redis_inst, *z_callargs, *zp_tmp;
+	zval z_fun, *redis_inst, *z_callargs, *zp_tmp;
 	char *key = NULL; /* set to avoid "unused-but-set-variable" */
 	int i, key_len = 0, argc;
 	HashTable *h_args;
@@ -342,10 +358,12 @@ ra_forward_call(INTERNAL_FUNCTION_PARAMETERS, RedisArray *ra, const char *cmd, i
 		redis_inst = ra->z_multi_exec; /* we already have the instance */
 	} else {
 		/* extract key and hash it. */
-		if(!(key = ra_find_key(ra, z_args, cmd, &key_len))) {
+        if ((zp_tmp = zend_hash_index_find(h_args, 0)) == NULL || Z_TYPE_P(zp_tmp) != IS_STRING) {
 			php_error_docref(NULL TSRMLS_CC, E_ERROR, "Could not find key");
 			RETURN_FALSE;
-		}
+        }
+        key = Z_STRVAL_P(zp_tmp);
+        key_len = Z_STRLEN_P(zp_tmp);
 
 		/* find node */
 		redis_inst = ra_find_node(ra, key, key_len, NULL TSRMLS_CC);
@@ -355,13 +373,6 @@ ra_forward_call(INTERNAL_FUNCTION_PARAMETERS, RedisArray *ra, const char *cmd, i
 		}
 	}
 
-	/* check if write cmd */
-	b_write_cmd = ra_is_write_cmd(ra, cmd, cmd_len);
-
-	if(ra->index && b_write_cmd && !ra->z_multi_exec) { /* add MULTI + SADD */
-		ra_index_multi(redis_inst, MULTI TSRMLS_CC);
-	}
-
 	/* pass call through */
 	ZVAL_STRINGL(&z_fun, cmd, cmd_len); /* method name */
 	z_callargs = ecalloc(argc + 1, sizeof(zval));
@@ -375,18 +386,23 @@ ra_forward_call(INTERNAL_FUNCTION_PARAMETERS, RedisArray *ra, const char *cmd, i
 
 	/* multi/exec */
 	if(ra->z_multi_exec) {
-		call_user_function(&redis_ce->function_table, ra->z_multi_exec, &z_fun, &z_tmp, argc, z_callargs);
+        call_user_function(&redis_ce->function_table, ra->z_multi_exec, &z_fun, return_value, argc, z_callargs);
+        zval_dtor(return_value);
         zval_dtor(&z_fun);
-        zval_dtor(&z_tmp);
 		efree(z_callargs);
 		RETURN_ZVAL(getThis(), 1, 0);
 	}
 
+    /* check if write cmd */
+    b_write_cmd = ra_is_write_cmd(ra, cmd, cmd_len);
+
 	/* CALL! */
 	if(ra->index && b_write_cmd) {
+        /* add MULTI + SADD */
+        ra_index_multi(redis_inst, MULTI TSRMLS_CC);
 		/* call using discarded temp value and extract exec results after. */
-		call_user_function(&redis_ce->function_table, redis_inst, &z_fun, &z_tmp, argc, z_callargs);
-		zval_dtor(&z_tmp);
+        call_user_function(&redis_ce->function_table, redis_inst, &z_fun, return_value, argc, z_callargs);
+        zval_dtor(return_value);
 
 		/* add keys to index. */
 		ra_index_key(key, key_len, redis_inst TSRMLS_CC);
@@ -432,7 +448,7 @@ PHP_METHOD(RedisArray, __call)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -450,7 +466,7 @@ PHP_METHOD(RedisArray, _hosts)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -474,7 +490,7 @@ PHP_METHOD(RedisArray, _target)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -499,7 +515,7 @@ PHP_METHOD(RedisArray, _instance)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -521,7 +537,7 @@ PHP_METHOD(RedisArray, _function)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -539,7 +555,7 @@ PHP_METHOD(RedisArray, _distributor)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -559,7 +575,7 @@ PHP_METHOD(RedisArray, _rehash)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -581,7 +597,7 @@ static void multihost_distribute(INTERNAL_FUNCTION_PARAMETERS, const char *metho
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -650,7 +666,7 @@ PHP_METHOD(RedisArray, keys)
 	}
 
 	/* Make sure we can grab our RedisArray object */
-	if(redis_array_get(object, &ra TSRMLS_CC) < 0) {
+    if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -693,7 +709,7 @@ PHP_METHOD(RedisArray, getOption)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -732,7 +748,7 @@ PHP_METHOD(RedisArray, setOption)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -771,7 +787,7 @@ PHP_METHOD(RedisArray, select)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -858,7 +874,7 @@ PHP_METHOD(RedisArray, mget)
 	HashTable *h_keys;
 	zval **argv;
 
-    if (redis_array_get(getThis(), &ra TSRMLS_CC) < 0) {
+    if ((ra = redis_array_get(getThis() TSRMLS_CC)) == NULL) {
         RETURN_FALSE;
     }
 
@@ -1002,120 +1018,124 @@ PHP_METHOD(RedisArray, mget)
 /* MSET will distribute the call to several nodes and regroup the values. */
 PHP_METHOD(RedisArray, mset)
 {
-	zval *object, *z_keys, z_argarray, *data, z_ret, **argv;
-	int i, n;
-	RedisArray *ra;
-	int *pos, argc, *argc_each;
-	HashTable *h_keys;
+    zval *object, *z_keys, z_argarray, *data, z_ret, **argv;
+    int i = 0, n;
+    RedisArray *ra;
+    int *pos, argc, *argc_each;
+    HashTable *h_keys;
     char *key, **keys, kbuf[40];
     int key_len, *key_lens;
     zend_string *zkey;
     ulong idx;
 
-    if (redis_array_get(getThis(), &ra TSRMLS_CC) < 0) {
+    if ((ra = redis_array_get(getThis() TSRMLS_CC)) == NULL) {
         RETURN_FALSE;
     }
 
-	/* Multi/exec support */
+    /* Multi/exec support */
     HANDLE_MULTI_EXEC(ra, "MSET");
 
-	if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Oa",
-				&object, redis_array_ce, &z_keys) == FAILURE) {
-		RETURN_FALSE;
-	}
+    if (zend_parse_method_parameters(ZEND_NUM_ARGS() TSRMLS_CC, getThis(), "Oa",
+                                     &object, redis_array_ce, &z_keys) == FAILURE)
+    {
+        RETURN_FALSE;
+    }
 
-	/* init data structures */
-	h_keys = Z_ARRVAL_P(z_keys);
-	argc = zend_hash_num_elements(h_keys);
-	argv = emalloc(argc * sizeof(zval*));
-	pos = emalloc(argc * sizeof(int));
-	keys = emalloc(argc * sizeof(char*));
-	key_lens = emalloc(argc * sizeof(int));
+    /* init data structures */
+    h_keys = Z_ARRVAL_P(z_keys);
+    argc = zend_hash_num_elements(h_keys);
+    argv = emalloc(argc * sizeof(zval*));
+    pos = emalloc(argc * sizeof(int));
+    keys = emalloc(argc * sizeof(char*));
+    key_lens = emalloc(argc * sizeof(int));
 
-	argc_each = emalloc(ra->count * sizeof(int));
-	memset(argc_each, 0, ra->count * sizeof(int));
+    argc_each = emalloc(ra->count * sizeof(int));
+    memset(argc_each, 0, ra->count * sizeof(int));
 
-	/* associate each key to a redis node */
-    i = 0;
+    /* associate each key to a redis node */
     ZEND_HASH_FOREACH_KEY_VAL(h_keys, idx, zkey, data) {
-	    /* If the key isn't a string, make a string representation of it */
+        /* If the key isn't a string, make a string representation of it */
         if (zkey) {
             key_len = zkey->len;
             key = zkey->val;
         } else {
-	        key_len = snprintf(kbuf, sizeof(kbuf), "%lu", idx);
+            key_len = snprintf(kbuf, sizeof(kbuf), "%lu", idx);
             key = kbuf;
         }
 
         if (ra_find_node(ra, key, (int)key_len, &pos[i] TSRMLS_CC) == NULL) {
             // TODO: handle
         }
-		argc_each[pos[i]]++;	/* count number of keys per node */
-		keys[i] = estrndup(key, key_len);
-		key_lens[i] = (int)key_len;
-		argv[i] = data;
+
+        argc_each[pos[i]]++;	/* count number of keys per node */
+        keys[i] = estrndup(key, key_len);
+        key_lens[i] = (int)key_len;
+        argv[i] = data;
         i++;
-	} ZEND_HASH_FOREACH_END();
+    } ZEND_HASH_FOREACH_END();
 
 
-	/* calls */
-	for(n = 0; n < ra->count; ++n) { /* for each node */
-	    /* We don't even need to make a call to this node if no keys go there */
-	    if(!argc_each[n]) continue;
+    /* calls */
+    for (n = 0; n < ra->count; ++n) { /* for each node */
+        /* We don't even need to make a call to this node if no keys go there */
+        if(!argc_each[n]) continue;
 
-		int found = 0;
+        int found = 0;
+
+        /* copy args */
+        array_init(&z_argarray);
+        for(i = 0; i < argc; ++i) {
+            if(pos[i] != n) continue;
 
-		/* copy args */
-		array_init(&z_argarray);
-		for(i = 0; i < argc; ++i) {
-			if(pos[i] != n) continue;
             zval zv, *z_tmp = &zv;
 #if (PHP_MAJOR_VERSION < 7)
             MAKE_STD_ZVAL(z_tmp);
 #endif
             ZVAL_ZVAL(z_tmp, argv[i], 1, 0);
-			add_assoc_zval_ex(&z_argarray, keys[i], key_lens[i], z_tmp);
-			found++;
-		}
+            add_assoc_zval_ex(&z_argarray, keys[i], key_lens[i], z_tmp);
+            found++;
+        }
 
-		if(!found)
-		{
-			zval_dtor(&z_argarray);
-			continue;				/* don't run empty MSETs */
-		}
+        if(!found) {
+            zval_dtor(&z_argarray);
+            continue; /* don't run empty MSETs */
+        }
 
-		if(ra->index) { /* add MULTI */
-			ra_index_multi(&ra->redis[n], MULTI TSRMLS_CC);
-		}
+        if(ra->index) { /* add MULTI */
+            ra_index_multi(&ra->redis[n], MULTI TSRMLS_CC);
+        }
 
         zval z_fun;
+
         /* prepare call */
         ZVAL_STRINGL(&z_fun, "MSET", 4);
-		/* call */
+
+        /* call */
         call_user_function(&redis_ce->function_table, &ra->redis[n], &z_fun, &z_ret, 1, &z_argarray);
-		zval_dtor(&z_fun);
-		zval_dtor(&z_ret);
+        zval_dtor(&z_fun);
+        zval_dtor(&z_ret);
 
-		if(ra->index) {
-			ra_index_keys(&z_argarray, &ra->redis[n] TSRMLS_CC); /* use SADD to add keys to node index */
-			ra_index_exec(&ra->redis[n], NULL, 0 TSRMLS_CC); /* run EXEC */
-		}
-		zval_dtor(&z_argarray);
-	}
+        if(ra->index) {
+            ra_index_keys(&z_argarray, &ra->redis[n] TSRMLS_CC); /* use SADD to add keys to node index */
+            ra_index_exec(&ra->redis[n], NULL, 0 TSRMLS_CC); /* run EXEC */
+        }
 
-	/* Free any keys that we needed to allocate memory for, because they weren't strings */
-	for(i = 0; i < argc; i++) {
-	    efree(keys[i]);
-	}
+        zval_dtor(&z_argarray);
+    }
 
-	/* cleanup */
-	efree(keys);
-	efree(key_lens);
-	efree(argv);
-	efree(pos);
-	efree(argc_each);
+    /* Free any keys that we needed to allocate memory for, because they weren't strings */
+    for(i = 0; i < argc; i++) {
+        efree(keys[i]);
+    }
 
-	RETURN_TRUE;
+    /* cleanup */
+    efree(keys);
+    efree(key_lens);
+    efree(argv);
+    efree(pos);
+    efree(argc_each);
+
+    RETURN_TRUE;
 }
 
 /* DEL will distribute the call to several nodes and regroup the values. */
@@ -1130,7 +1150,7 @@ PHP_METHOD(RedisArray, del)
 	long total = 0;
 	int free_zkeys = 0;
 
-    if (redis_array_get(getThis(), &ra TSRMLS_CC) < 0) {
+    if ((ra = redis_array_get(getThis() TSRMLS_CC)) == NULL) {
         RETURN_FALSE;
     }
 	/* Multi/exec support */
@@ -1271,7 +1291,7 @@ PHP_METHOD(RedisArray, multi)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL) {
 		RETURN_FALSE;
 	}
 
@@ -1305,7 +1325,7 @@ PHP_METHOD(RedisArray, exec)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0 || !ra->z_multi_exec) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL || !ra->z_multi_exec) {
 		RETURN_FALSE;
 	}
 
@@ -1326,7 +1346,7 @@ PHP_METHOD(RedisArray, discard)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0 || !ra->z_multi_exec) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL || !ra->z_multi_exec) {
 		RETURN_FALSE;
 	}
 
@@ -1347,7 +1367,7 @@ PHP_METHOD(RedisArray, unwatch)
 		RETURN_FALSE;
 	}
 
-	if (redis_array_get(object, &ra TSRMLS_CC) < 0 || !ra->z_multi_exec) {
+	if ((ra = redis_array_get(object TSRMLS_CC)) == NULL || !ra->z_multi_exec) {
 		RETURN_FALSE;
 	}
 
